#pragma once

#include <a8/singleton.h>

#include <f8/app.h>

namespace a8
{
    struct UdpPacket;
}

struct MsgNode;
struct UdpMsgNode;
class App : public a8::Singleton<App>, public f8::UserApp
{
private:
    App() {};
    friend class a8::Singleton<App>;

public:

    virtual const std::string GetPkgName() override;
    virtual void Init() override;
    virtual void UnInit() override;
    virtual void Update(int delta_time) override;
    virtual bool HasTask() override;
    virtual void DispatchSocketMsg(f8::MsgHdr* hdr) override;

    void AddUdpMsg(a8::UdpPacket* pkt);

    PerfMonitor& GetPerf() { return perf_; }
    int GetUdpMsgNodeSize() { return udp_msgnode_size_;}

    int GetGameId() const;

private:
    void QuickExecute();
    void SlowerExecute();

    void DispatchUdpMsg();

    void ProcessClientMsg(f8::MsgHdr* hdr);
    void ProcessMasterServerMsg(f8::MsgHdr* hdr);
    void ProcessTargetServerMsg(f8::MsgHdr* hdr);

    void FreeUdpMsgQueue();

private:
    PerfMonitor perf_;

    std::shared_ptr<std::mutex> udp_msg_mutex_;
    UdpMsgNode* udp_top_node_ = nullptr;
    UdpMsgNode* udp_bot_node_ = nullptr;
    UdpMsgNode* udp_work_node_ = nullptr;

    int udp_msgnode_size_ = 0 ;
    int udp_working_msgnode_size_ = 0;

};
